
# q-expansion of newform 38.6.a.d, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 38
weight = 6
poly_data = [3760, -454, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-306, 11, 1], 2]]

hecke_ring_character_values = None
aps_data = [[4, 0, 0], [4, 1, 0], [27, -1, 1], [79, -4, -5], [121, -17, 17], [178, -5, -28], [-429, 60, 21], [-361, 0, 0], [-318, -157, 34], [-2844, 121, 62], [-2388, 28, -196], [-510, -236, 116], [3478, -66, -228], [913, 549, 489], [11055, 71, -5], [10106, 867, -156], [5920, 2405, 244], [5779, -623, 401], [-610, 2095, -1054], [7326, -1996, -818], [-24541, -1898, -739], [22212, 758, 964], [600, 4278, 684], [-29512, 4384, -1354], [33168, 2218, 3302], [-26050, -132, -72], [89174, -1892, 1238], [-26840, -2933, -868], [48810, -663, 4284], [-134586, 80, -1148], [128698, -4120, 658], [-102441, 11235, 10623], [32451, -10568, -3667], [251045, 901, -4339], [-212011, 8029, 1235], [-15154, -12396, -7986], [-90610, -7360, -1136], [197300, -10500, 17736], [252986, 382, -7606], [68558, -9308, -19336], [471074, -5246, -6610], [-334262, -24396, -4404], [-241165, 1004, -1241], [-290412, -21102, 2970], [558174, -4714, 12568], [-25165, -3756, -16929], [-561388, 25909, 18944], [-624770, 1312, -5626], [-378290, 16563, -11790], [-440119, 15315, 5463], [-91413, -14967, 20157], [808639, 15272, -19613], [-498206, -32406, -12816], [-920337, 23813, -17297], [562324, 33700, 15770], [448635, -52557, 4191], [-1090624, 28498, -14158], [-258174, 21835, -36358], [1503277, 50409, 5163], [106426, 60426, 33744], [-220239, -27417, 23049], [351222, -16851, 24672], [-1272252, -43056, -30312], [472625, -91966, -21611], [667936, 3511, -37450], [-1289288, -26797, -16862], [-98118, -77467, -28442], [120628, 35916, -1686], [1397407, 16929, 46095], [-644555, -36861, 58395], [-812628, 77247, 55050], [-138069, 62058, -38457], [2629096, -19020, 35856], [-2234634, -58327, 60976], [1742972, 841, -59152], [684800, -70966, 122800], [-1905973, 132721, 24161], [1483529, -108231, 1983], [473496, -215000, -32914], [3145376, 90788, 46510], [3496036, -15868, 90904], [518350, 36499, 20408], [3234140, -36626, -11764], [-52430, -25340, -2224], [-367790, -238658, 17330], [-1081273, 117545, 82231], [-1850992, -103426, -17402], [5354877, 15632, -59249], [-1313255, -378011, 73775], [63611, -130921, -110585], [-5045273, -21039, 20895], [3683900, 92552, -132812], [-522502, -51700, -56390], [3598516, 338368, 43184], [-1929765, 179611, -75229], [-3004826, -44713, -223418], [-265464, -360234, 14346], [554900, 91152, 7650], [3907384, -268433, -128644], [-4473685, 184809, 203721], [-1835364, 302320, 141392], [-6273043, 60737, 160243], [-523982, 148454, 105214], [-2448036, -206972, -211366], [802084, 74872, -65392], [638233, 79834, 15923], [2302243, -17545, -285389], [-8897882, -300060, -92796], [3691500, 156444, 329940], [3234142, 169686, -128316], [-3028468, -348198, 108780], [4181377, -223857, 455631], [7344145, 285767, -188981], [-2665080, -492266, 212036], [6345833, -208503, 66981], [2923856, -210890, 261926], [-1935471, 34723, -333823], [2791475, -49932, 145023], [-5289615, 572733, 478119], [2804300, 3003, -11640], [8993922, -128615, -327700], [-868078, 603174, -69840], [4007504, 663299, 188050], [-3956132, -463704, 349224], [-2707981, -373751, -326437], [3490122, -419218, 304396], [690754, -585016, -31604], [-4636945, 417572, -230333], [2216881, -257958, 612405], [-3019682, 917930, 294688], [-14906307, 80237, 80701], [17761216, -226472, -337096], [-10917200, 217720, -97552], [-125741, 257013, 598329], [-4477505, -149992, -215255], [-10478271, 446176, 50795], [-12684268, 54331, -591706], [638078, -737629, 414034], [-3672214, -33929, -317908], [-1097167, -140348, -563821], [-6631490, -226843, 436618], [-11654839, -775643, 259799], [1428819, 1321094, -150785], [-7984394, -1276709, 296450], [9853970, 647377, -131788], [3822446, 248634, 268638], [11571314, 682672, -76900], [-5199186, -1047054, 599940], [30630765, 93583, -47467], [13336322, 368140, -312934], [1074108, -680009, -708154], [-20262579, 758391, 173103], [4981909, 185619, 159381], [-3890182, -144820, -721238], [13148318, -410115, -671574], [-13699306, 704410, 110462], [-7154270, 52165, 1051538], [-796464, 190523, -415010], [14343195, -153562, -174059], [-14446036, 18173, -572138], [-22865468, 983504, 62440], [-2836008, -729994, 230806], [-5773968, -365208, -9120], [5129876, -1428192, -189600], [7335044, 340284, 12714], [13669152, -306618, 907560], [11768802, 304906, 170426], [17555441, 258173, -697361]]
